#include "stdafx.h"
#include "cel.h"
#include "resource.h"

const UINT SC_SAVE = 0xF800;
const UINT SC_ABOUT = 0xF810;
const int MIN_WIDTH = 120;
const MIN_HEIGHT = 20;
const char* caption = "CelView";
const char* cel_auto_file = "cel_auto_file";
const char* cel_file = "Cel File";

bool center = true;
cel image;

bool open_filename (HWND hWnd, char* buf)
{
	OPENFILENAME ofn;

	memset (&ofn, 0, sizeof (ofn));
	ofn.lStructSize = sizeof (OPENFILENAME);
	ofn.hwndOwner = hWnd;
	ofn.hInstance = GetModuleHandle (NULL);
	ofn.lpstrFile = buf;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrFilter = "Cel files (*.cel)\0*.cel\0All files (*.*)\0*.*\0\0";
	ofn.Flags = OFN_FILEMUSTEXIST | OFN_NOCHANGEDIR |
		OFN_PATHMUSTEXIST | OFN_HIDEREADONLY;

	return (GetOpenFileName (&ofn) != 0);
}

bool save_filename (HWND hWnd, char* buf)
{
	OPENFILENAME ofn;

	memset (&ofn, 0, sizeof (ofn));
	ofn.lStructSize = sizeof (OPENFILENAME);
	ofn.hwndOwner = hWnd;
	ofn.hInstance = GetModuleHandle (NULL);
	ofn.lpstrFile = buf;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrFilter = "Bitmap files (*.bmp)\0*.bmp\0All files (*.*)\0*.*\0\0";
	ofn.Flags = OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;

	return (GetSaveFileName (&ofn) != 0);
}

LRESULT CALLBACK WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static HINSTANCE hInst = NULL;
	static HBRUSH cross = NULL;

	switch (message) {
		case WM_CREATE:
			{
				LPCREATESTRUCT lpcs = (LPCREATESTRUCT) lParam;
				hInst = lpcs->hInstance;
				cross = CreateHatchBrush (HS_DIAGCROSS, RGB (255, 200, 200));
				HMENU menu = GetSystemMenu (hWnd, FALSE);
				AppendMenu (menu, MF_SEPARATOR, SC_SEPARATOR, NULL);
				AppendMenu (menu, MF_STRING, SC_SAVE, "Save bitmap...");
				AppendMenu (menu, MF_STRING, SC_ABOUT, "About...");
			}
			return 0L ;

		case WM_GETMINMAXINFO:
			{
				MINMAXINFO *mmi = (MINMAXINFO*) lParam;
				RECT rc;
				rc.left = 0;
				rc.right = MIN_WIDTH - 1;
				rc.top = 0;
				rc.bottom = MIN_HEIGHT - 1;
				AdjustWindowRectEx (&rc, GetWindowLong (hWnd, GWL_STYLE),
					FALSE, GetWindowLong (hWnd, GWL_EXSTYLE));
				mmi->ptMinTrackSize.x = rc.right - rc.left + 1;
				mmi->ptMinTrackSize.y = rc.bottom - rc.top + 1;
			}
			return 0L;

		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				BeginPaint (hWnd, &ps);

				RECT rc;
				GetClientRect (hWnd, &rc);

				if (center) {
					FillRect (ps.hdc, &rc, cross);
					image.draw (ps.hdc,
						rc.left + (rc.right - rc.left - image.width ()) / 2,
						rc.top + (rc.bottom - rc.top - image.height ()) / 2);
				} else
					image.draw (ps.hdc, rc.left, rc.top,
						rc.right - rc.left, rc.bottom - rc.top);

				EndPaint (hWnd, &ps);
			}
			return 0L;

		case WM_SYSCOMMAND:
			switch (wParam & 0xFFF0) {
				case SC_SAVE:
					char output [MAX_PATH];
					strcpy (output, image.get_name ());
					strcat (output, ".bmp");
					if (save_filename (hWnd, output))
						image.save (output);
					return 0L;

				case SC_ABOUT:
					MessageBox (hWnd, "CelView 1.0.0.3\n"
						"Copyright (c) 2001-02 Nikolay Raspopov\n"
						"mail: ryo-oh-ki@narod.ru\n"
						"www:  http://raspopov.dem.ru/", caption,
						MB_OK | MB_ICONINFORMATION);
					return 0L;
			}
			break;

		case WM_LBUTTONDBLCLK:
			center = !center;
			InvalidateRect (hWnd, NULL, FALSE);
			UpdateWindow (hWnd);
			return 0L;

		case WM_DESTROY:
			DeleteObject (cross);
			PostQuitMessage (0);
			return 0L;
	}
	return DefWindowProc (hWnd, message, wParam, lParam);
}

int APIENTRY WinMain (HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow)
{
	char buf [MAX_PATH + 10];

	HKEY key;
	RegCreateKeyEx (HKEY_CLASSES_ROOT,
		".cel", 0, NULL, REG_OPTION_NON_VOLATILE,
		KEY_WRITE, NULL, &key, NULL);
	RegSetValueEx (key, NULL, 0, REG_SZ,
		(const unsigned char*) cel_auto_file, strlen (cel_auto_file) + 1);
	RegCloseKey (key);

	RegCreateKeyEx (HKEY_CLASSES_ROOT,
		cel_auto_file, 0, NULL, REG_OPTION_NON_VOLATILE,
		KEY_WRITE, NULL, &key, NULL);
	RegSetValueEx (key, NULL, 0, REG_SZ,
		(const unsigned char*) cel_file, strlen (cel_file) + 1);
	
	HKEY sub_key;
	RegCreateKeyEx (key, "DefaultIcon", 0, NULL, REG_OPTION_NON_VOLATILE,
		KEY_WRITE, NULL, &sub_key, NULL);
	strcpy (buf, __argv [0]);
	strcat (buf, ",1");
	RegSetValueEx (sub_key, NULL, 0, REG_SZ,
		(const unsigned char*) buf, strlen (buf) + 1);
	RegCloseKey (sub_key);

	RegCreateKeyEx (key,
		"shell\\open\\command", 0, NULL, REG_OPTION_NON_VOLATILE,
		KEY_WRITE, NULL, &sub_key, NULL);
	strcpy (buf, "\"");
	strcat (buf, __argv [0]);
	strcat (buf, "\" \"%1\"");
	RegSetValueEx (sub_key, NULL, 0, REG_SZ,
		(const unsigned char*) buf, strlen (buf) + 1);
	RegCloseKey (sub_key);

	RegCloseKey (key);

	char input [MAX_PATH];
	*input = '\0';
	if (((__argc > 1 && strcpy (input, __argv [1])) ||
		open_filename (GetDesktopWindow (), input)) &&
		image.load (input)) {

		WNDCLASSEX wcx;
		memset (&wcx, 0, sizeof (wcx));
		wcx.style = CS_VREDRAW | CS_HREDRAW | CS_DBLCLKS;
		wcx.cbSize = sizeof (WNDCLASSEX);
		wcx.lpfnWndProc = WndProc;
		wcx.hInstance = hInst;
		wcx.hIcon = (HICON) LoadImage (hInst, MAKEINTRESOURCE (IDI_APP),
			IMAGE_ICON, 32, 32, 0);
		wcx.hIconSm = (HICON) LoadImage (hInst, MAKEINTRESOURCE (IDI_APP),
			IMAGE_ICON, 16, 16, 0);
		wcx.hCursor = LoadCursor (NULL, IDC_ARROW);
		wcx.lpszClassName = caption;
		if (RegisterClassEx (&wcx)) {

			DWORD exstyle = WS_EX_WINDOWEDGE;
			DWORD style = WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

			int width = image.width ();
			int height = image.height ();
			RECT rc;
			rc.left = 0;
			rc.right = width - 1;
			rc.top = 0;
			rc.bottom = height - 1;
			AdjustWindowRectEx (&rc, style, FALSE, exstyle);
			int win_width = rc.right - rc.left + 1;
			int win_height = rc.bottom - rc.top + 1;
			int x = (GetSystemMetrics (SM_CXSCREEN) - win_width) / 2;
			int y = (GetSystemMetrics (SM_CYSCREEN) - win_height) / 2;

			char title [100];
			wsprintf (title, "%s [%d x %d x %d bit] - %s",
				image.get_name (), image.width (), image.height (), image.colors (),
				caption);

			HWND hWnd = CreateWindowEx (exstyle, wcx.lpszClassName, title,
				style, x, y, win_width, win_height, NULL, NULL, hInst, 0);
			if (hWnd) {

				ShowWindow (hWnd, nCmdShow);
				UpdateWindow (hWnd);

				MSG msg;
				while (GetMessage (&msg, NULL, 0, 0)) {
					TranslateMessage (&msg);
					DispatchMessage (&msg);
				}
			}				
		}
	}

	return 0;

	(void) lpCmdLine;
	(void) hPrevInst;
}



