#include "stdafx.h"
#include "cel.h"

cel::cel() :
	name (""),
	dir (""),
	mark (0),
	fix (0),
	pal_num (0),
	bits (NULL),
	bitmap (NULL)
{
	memset (&header, 0, sizeof (header));
	memset (&bmi, 0, sizeof (bmi));
	bmi.bmiHeader.biSize = sizeof (BITMAPINFOHEADER);
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biBitCount = 24;
	bmi.bmiHeader.biCompression = BI_RGB;
	bmi.bmiHeader.biXPelsPerMeter = 2880;	// 72 dpi
	bmi.bmiHeader.biYPelsPerMeter = 2880;	// 72 dpi
}

cel::~cel()
{
	if (bitmap)
		DeleteObject (bitmap);
}

bool cel::load (const char* filename)
{
	bool ret = false;

	char cur_dir [MAX_PATH];
	GetCurrentDirectory (MAX_PATH, cur_dir);

	//   
	char *n = strrchr (filename, '\\');
	if (n) {
		name = n + 1;
		dir = filename;
		dir = dir.substr (0, n - filename);
		SetCurrentDirectory (dir.c_str ());
	} else {
		name = filename;
		dir = cur_dir;
	}

	FILE *in = fopen (name.c_str (), "rb");
	if (in) {
		//   
		if (1 == fread (&header, sizeof (header), 1, in) &&
			!memcmp (header.magic, "KiSS", 4) &&
			(header.tag == 0x20 || header.tag == 0x21)) {
			//  

			if (header.bits == 4 || header.bits == 8)
				//   -   
				load_palette ();

			ret = load_bitmap (in);
		} else {
			rewind (in);

			if (1 == fread (&(header.cx), 4, 1, in) &&
				memcmp (&(header.cx), "KiSS", 4)) {
				//  

				header.bits = 4;

				load_palette ();

				ret = load_bitmap (in);
			}
		}
		fclose (in);
	}

	SetCurrentDirectory (cur_dir);

	return ret;
}

bool cel::load_palette ()
{
	bool ret = false;

	//   
	WIN32_FIND_DATA wfa;
	HANDLE ff = FindFirstFile ("*.cnf", &wfa);
	if (ff != INVALID_HANDLE_VALUE) {
		do {
			if (!(wfa.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
				ret = load_config (wfa.cFileName);
		} while (!ret && FindNextFile (ff, &wfa));
		FindClose (ff); 
	}

	if (!ret) {
		//  
		ff = FindFirstFile ("*.kcf", &wfa);
		if (ff != INVALID_HANDLE_VALUE) {
			do {
				if (!(wfa.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
					ret = pal.load (wfa.cFileName);
			} while (!ret && FindNextFile (ff, &wfa));
			FindClose (ff); 
		}
	}

	return ret;
}

bool cel::load_config (const char* filename)
{
	//   
	bool ret = false;
	FILE* in = fopen (filename, "rb");
	if (in) {
		char buf [260];
		while (!feof (in) && fgets (buf, sizeof (buf), in) && !ret) {

			//  
			char *line = trimright (trimleft (buf));

			switch (*line) {
				case '%':
				//  
				pal.load (line + 1);
				break;

				case '#':
				//  
				ret = parse (line + 1);
				break;
			}
		}

		fclose (in);
	}
	return ret;
}

bool cel::parse (char* line)
{
	//   
	size_t n = strcspn (line, " \t");
	if (line [n]) {
		char *c = &line [n];
		*c = '\0';
		char *file_name = trimleft (c + 1);

		//  
		c = strchr (line, '.');
		if (c) {
			*c = '\0';
			fix = atoi (c + 1);
		}

		//  
		mark = atoi (line);

		pal_num = 0;
		n = strcspn (file_name, " \t");
		if (file_name [n]) {
			c = &file_name [n];
			*c = '\0';
			c = trimleft (c + 1);
			if (*c == '*')
				pal_num = atoi (c + 1);
		}
		return (pal_num >= 0) && !stricmp (name.c_str (), file_name);
	}
	return false;
}

bool cel::load_bitmap (FILE* in)
{
	bool ret = true;

	bmi.bmiHeader.biWidth = header.cx;
	bmi.bmiHeader.biHeight = header.cy;
	bmi.bmiHeader.biSizeImage = size ();

	HDC hdc = GetDC (NULL);
	bitmap = CreateDIBSection (hdc,
		(BITMAPINFO*) &bmi, DIB_RGB_COLORS, (void**) &bits, NULL, 0); 
	ReleaseDC (NULL, hdc);

	switch (header.bits) {
		case 4:
		{
			unsigned char buf = 0;
			for (int y = 0; y < header.cy && ret; y++) {
				bool t = false;
				for (int x = 0; x < header.cx && ret; x++) {
					if (t)
						buf <<= 4;
					else
						ret = (1 == fread (&buf, sizeof (buf), 1, in));
					t = !t;
					char *a = get_pixel_addr (x, header.cy - y - 1);
					unsigned int color_index = (buf & 0xf0) >> 4;
					*a++ = pal.blue (color_index, pal_num);
					*a++ = pal.green (color_index, pal_num);
					*a = pal.red (color_index, pal_num);
				}
			}
		}
		break;

		case 8:
		{
			unsigned char buf;
			for (int y = 0; y < header.cy && ret; y++) {
				for (int x = 0; x < header.cx && ret; x++) {
					ret = (1 == fread (&buf, sizeof (buf), 1, in));
					char *a = get_pixel_addr (x, header.cy - y - 1);
					unsigned int color_index = buf;
					*a++ = pal.blue (color_index, pal_num);
					*a++ = pal.green (color_index, pal_num);
					*a = pal.red (color_index, pal_num);
				}
			}
		}
		break;

		case 16:
		{
			unsigned short buf;
			for (int y = 0; y < header.cy && ret; y++) {
				for (int x = 0; x < header.cx && ret; x++) {
					ret = (1 == fread (&buf, sizeof (buf), 1, in));
					char *a = get_pixel_addr (x, header.cy - y - 1);
					*a++ = (char) ((buf & 0x1f) << 3);
					*a++ = (char) (((buf >> 5) & 0x1f) << 3);
					*a = (char) (((buf >> 10) & 0x1f) << 3);
				}
			}
		}
		break;

		case 24:
		{
			unsigned char buf [3];
			for (int y = 0; y < header.cy && ret; y++) {
				for (int x = 0; x < header.cx && ret; x++) {
					ret = (1 == fread (&buf, sizeof (buf), 1, in));
					char *a = get_pixel_addr (x, header.cy - y - 1);
					*a++ = buf [0];
					*a++ = buf [1];
					*a = buf [2];
				}
			}
		}
		break;

		case 32:
		{
			unsigned char buf [4];
			for (int y = 0; y < header.cy && ret; y++) {
				for (int x = 0; x < header.cx && ret; x++) {
					ret = (1 == fread (&buf, sizeof (buf), 1, in));
					char *a = get_pixel_addr (x, header.cy - y - 1);
					*a++ = buf [0];
					*a++ = buf [1];
					*a = buf [2];
				}
			}
		}
		break;

		default:
			ret = false;
	}

	return ret;
}

void cel::draw (HDC hdc, int x, int y, int cx, int cy)
{
	if (bitmap) {
		SetStretchBltMode (hdc, COLORONCOLOR);
		StretchDIBits (hdc,
			x, y, cx ? cx : header.cx, cy ? cy : (int) header.cy,
			0, 0, header.cx, header.cy,
			bits, (BITMAPINFO*) &bmi, DIB_RGB_COLORS, SRCCOPY);
	} else
		Rectangle (hdc, x, y, x + cx - 1, y + cy - 1);
}

bool cel::save (const char *filename)
{
	bool ret = false;
	FILE *out = fopen (filename, "wb");
	if (out) {

		BITMAPFILEHEADER hdr;
		hdr.bfType = 0x4d42;
		hdr.bfSize = (DWORD) (sizeof (BITMAPFILEHEADER) +
			bmi.bmiHeader.biSize + bmi.bmiHeader.biSizeImage);
		hdr.bfReserved1 = 0;
		hdr.bfReserved2 = 0;
		hdr.bfOffBits = (DWORD) (sizeof (BITMAPFILEHEADER) +
			bmi.bmiHeader.biSize);

		fwrite (&hdr, sizeof (BITMAPFILEHEADER), 1, out);
		fwrite (&(bmi.bmiHeader), sizeof (BITMAPINFOHEADER), 1, out);
		fwrite (bits, bmi.bmiHeader.biSizeImage, 1, out);

		fclose (out);
	}
	return ret;
}
