#if !defined(AFX_CEL_H__D1696459_CF62_11D5_8836_00C0DFF8C897__INCLUDED_)
#define AFX_CEL_H__D1696459_CF62_11D5_8836_00C0DFF8C897__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "palette.h"

#pragma pack(push,1)
typedef struct {
	unsigned char magic [4];
	unsigned char tag;
	unsigned char bits;
	unsigned short ext;
	unsigned short cx;
	unsigned short cy;
	unsigned short x;
	unsigned short y;
	unsigned char reserved [16];
} cel_header;
#pragma pack(pop)

class cel  
{
public:
	cel ();
	virtual ~cel ();

	bool load (const char *filename);	//  
	bool save (const char *filename);	//  

	void draw (HDC hdc, int x = 0, int y = 0,
		int cx = 0, int cy = 0);		//  

	inline const char* get_name () const {
		return name.c_str ();
	}
	inline int width () const {			// pixels
		return header.cx;
	}
	inline int height () const {		// pixels
		return header.cy;
	}
	inline int colors () const {
		return header.bits;
	}

protected:
	string name;				//  
	string dir;					//  
	int mark;					// 
	int fix;					// 
	int pal_num;				//  
	palette pal;				// 
	cel_header header;			//  
	BITMAPINFO bmi;				//  
	HBITMAP bitmap;				//  
	char *bits;					//   

	bool load_palette ();						//    
	bool load_config (const char* filename);	//   
	bool parse (char* line);					//   
	bool load_bitmap (FILE* in);				//    

	inline char* get_pixel_addr (const int x, const int y) const {
		return get_line_addr (y) + x * 3;
	}
	inline char* get_line_addr (const int y) const {
		return bits + y * line_width ();
	}
	inline int size () const {			// bytes
		return height () * line_width ();
	}
	inline int line_width () const {	// bytes
		return (((24 * width ()) + 31) / 32) * 4;
	}
};

#endif // !defined(AFX_CEL_H__D1696459_CF62_11D5_8836_00C0DFF8C897__INCLUDED_)
