#include "stdafx.h"
#include "palette.h"

palette::palette()
{
}

palette::~palette()
{
	for (color_vector::iterator i = groups.begin (); i != groups.end (); ++i)
		delete (*i);
	groups.clear ();
}

unsigned int palette::size ()
{
	unsigned int s = 0;
	for (color_vector::iterator i = groups.begin (); i != groups.end (); ++i)
		s += (*i)->size ();
	return s;
}

unsigned char palette::red (const int i, const int pal_num)
{
	int c = 0;
	color_vector::iterator j = groups.begin ();
	for (j += pal_num; j != groups.end (); ++j) {
		color_group* group = (*j);
		if (i >= c && i < c + group->size ())
			return group->red (i - c);
		c += group->size ();
	}
	return (unsigned char) (i << 4);
}

unsigned char palette::green (const int i, const int pal_num)
{
	int c = 0;
	color_vector::iterator j = groups.begin ();
	for (j += pal_num; j != groups.end (); ++j) {
		color_group* group = (*j);
		if (i >= c && i < c + group->size ())
			return group->green (i - c);
		c += group->size ();
	}
	return (unsigned char) (i << 4);
}

unsigned char palette::blue (const int i, const int pal_num)
{
	int c = 0;
	color_vector::iterator j = groups.begin ();
	for (j += pal_num; j != groups.end (); ++j) {
		color_group* group = (*j);
		if (i >= c && i < c + group->size ())
			return group->blue (i - c);
		c += group->size ();
	}
	return (unsigned char) (i << 4);
}

bool palette::load (const char* filename)
{
	bool ret = false;

	memset (&header, 0, sizeof (header));

	FILE *in = fopen (filename, "rb");
	if (in) {
		//   
		if (1 == fread (&header, sizeof (header), 1, in) &&
			!memcmp (header.magic, "KiSS", 4) &&
			header.tag == 0x10 &&
			(header.bits == 12 || header.bits == 24) &&
			header.groups >= 1 && header.groups <= 10 &&
			header.colors >= 1 && header.colors <= 256) {
			//  

		} else {
			//  
			rewind (in);

			header.bits = 12;
			header.groups = 10;
			header.colors = 16;
		}

		//   
		for (int i = 0; i < header.groups; i++) {
			color_group *group = new color_group ();
			if (group) {
				ret = group->load (in, header.colors, header.bits);
				if (!ret)
					break;
				groups.push_back (group);
			}
		}

		fclose (in);
	}

	return ret;
}
